% Create and navigate ortho views of Analyze 7.5 AVW volume image
%
% function [ varargout ] = avw_view(avw,parent,command,options)
%
%
% Petr Krysl: added options
%
% avw - a struct, created by avw_read; if omitted, a gui file locator
% will
% prompt for a .hdr file.
%
% parent  - an optional handle to the gui that calls this gui, useful for
% updating the UserData field of the parent.
% command - an optional string, such as 'init' or various calback
% commands, 'init' is the default.
%
% The navigation is by sliders, mouse clicks and arrow keys.  Right mouse
% clicks on any ortho-view will show a command menu, for  image zoom.
%
% The points are given in several coordinate
% frameworks (voxels, mm, and meters), with values given relative to an
% origin located at the "center" of the MRI volume (see avw_center, which
% returns abs and corner values, abs used here).
%
% The AC location can be selected and the values are returned into 'avw.ac'
% in the base workspace. These points are given in voxels, mm & meters; for
% the latter, the values are given as offsets from the "center" of the MRI
% volume (see avw_center).
%
% +X is left (L), +Y is anterior (A), +Z is superior (S), the default LAS
% orientation of the Analyze 7.5 format.  The coordinate system is left
% handed.  This is the radiological convention, as opposed to the
% neurological convention (RAS).  The latter can be emulated by using the
% 'Flip L/R' button.
%
% Example of loading and viewing the SPM T1 template:
% avw = avw_read('T1')
% avw = avw_view(avw);
%
% Similarly, just 'avw_view' can be typed at the command prompt and you can
% use the gui file locator to select any .hdr file.
%
% See also, avw_read, avw_img_read, avw_hdr_read
%

% $Revision: 1.1 $ $Date: 2008/04/04 15:39:46 $

% Licence:  GNU GPL, no express or implied warranties
% History:  06/2002, Darren.Weber_at_flinders.edu.au
%           10/2002, Darren.Weber_at_flinders.edu.au
%                    added fiducial point determination
%                    changed plots from surf to imagesc commands
%                    added handling of datatype for avw.img
%                    altered daspect to use avw.hdr.dime.pixdim
%                    altered color scheme
%           01/2003, Darren.Weber_at_flinders.edu.au
%                    added parent GUI handling
%           10/2003, Darren.Weber_at_radiology.ucsf.edu
%                    added right click options, including simple block ROI
%                    functions, zoom and save image
%           11/2003, Darren.Weber_at_radiology.ucsf.edu
%                    added arrow key navigation
%           2007-9   Petr Krysl, multiple changes
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [ varargout ] = avw_view(avw,parent,command,options)
    
    if gcbf,
        AVWVIEW = get(gcbf,'Userdata');
    end
    
    if ~exist('command','var'), command = 'init'; end
    
    command = lower(command);
    
    % Check for specific keys and assign command
    if strcmp(command, 'keypress'),
        
        cc = get(AVWVIEW.gui,'CurrentCharacter');
        cc = double(cc);
        if cc,
            switch cc,
                case 27, command = 'quit';  % ESC
                case 28, command = 'left';  % left
                case 29, command = 'right'; % right
                case 30, command = 'up';    % up
                case 31, command = 'down';  % down
                otherwise, return;  % all other keys
            end
        end
    end
    
    switch command,
        
        case 'init',
            
            if ~exist('avw','var'),
                avw = avw_read;
            end
            
            if ~exist('options','var'),
                options = struct('name',[]);
            end
            
            %         if nargin == 0,
            %             AVWVIEW = init(avw);
            %         elseif isempty(inputname(1)),
            %             AVWVIEW = init(avw);
            %         else
            %             AVWVIEW = init(avw,inputname(1));
            %         end
            % Petr Krysl
            if nargin == 0,
                options.name = [];
            elseif isempty(inputname(1)),
                options.name = [];
            else
                options.name = inputname(1);
            end
            AVWVIEW = init(avw,options);
            if exist('parent','var'), %Petr Krysl
                ud=set(parent,'Userdata');
                ud.current_avw_gui =AVWVIEW.gui;
                set(parent,'Userdata',ud);
            end
            AVWVIEW.view = 'axi'; axial_update(AVWVIEW)
            AVWVIEW.view = 'cor'; coronal_update(AVWVIEW)
            AVWVIEW.view = 'sag'; sagittal_update(AVWVIEW)
            
            
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            
        case 'coordinates',
            
            AVWVIEW = set_coordinates(AVWVIEW);
            
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            
        case 'poi',
            
            list = get(AVWVIEW.handles.listbox_poi,'String');
            wpoi =get(AVWVIEW.handles.listbox_poi,'Value');
            xyz=get(AVWVIEW.pois(wpoi),'xyz');
            Voxel =avw_xyz2voxel(AVWVIEW.avw,xyz);
            AVWVIEW.slices.sag= Voxel(1); AVWVIEW.slices.cor= Voxel(2); AVWVIEW.slices.axi= Voxel(3);
            AVWVIEW.view = 'axi';
            axial_update(AVWVIEW);
            AVWVIEW.view = 'cor';
            coronal_update(AVWVIEW);
            AVWVIEW.view = 'sag';
            sagittal_update(AVWVIEW);
            AVWVIEW = set_coordinates(AVWVIEW);
            
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            
        case 'crosshairs',
            
            AVWVIEW = set_crosshairs(AVWVIEW);
            
            
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            
        case {'axial_image','coronal_image','sagittal_image'},
            
            button = get(gcf, 'SelectionType');
            
            switch command,
                case 'axial_image',    AVWVIEW.view = 'axi'; axi_update = 0; cor_update = 1; sag_update = 1;
                case 'coronal_image',  AVWVIEW.view = 'cor'; axi_update = 1; cor_update = 0; sag_update = 1;
                case 'sagittal_image', AVWVIEW.view = 'sag'; axi_update = 1; cor_update = 1; sag_update = 0;
            end
            
            AVWVIEW = get_current_position(AVWVIEW);
            
            if (strcmp(button,'normal'))
                
                if axi_update,
                    axial_update(AVWVIEW);
                end
                if cor_update,
                    coronal_update(AVWVIEW);
                end;
                if sag_update,
                    sagittal_update(AVWVIEW);
                end;
                
                AVWVIEW = set_coordinates(AVWVIEW);
                %AVWVIEW = set_display_values(AVWVIEW);
                %AVWVIEW = set_crosshairs(AVWVIEW);
                
            end
            
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            
        case {'axial_slider','coronal_slider','sagittal_slider'},
            
            switch command,
                
                case 'axial_slider',
                    AVWVIEW.view = 'axi';
                    AVWVIEW = get_slider_position(AVWVIEW);
                    axial_update(AVWVIEW);
                    
                case 'coronal_slider',
                    AVWVIEW.view = 'cor';
                    AVWVIEW = get_slider_position(AVWVIEW);
                    coronal_update(AVWVIEW);
                    
                case 'sagittal_slider',
                    AVWVIEW.view = 'sag';
                    AVWVIEW = get_slider_position(AVWVIEW);
                    sagittal_update(AVWVIEW);
            end
            
            AVWVIEW = set_coordinates(AVWVIEW);
            %AVWVIEW = set_display_values(AVWVIEW);
            %AVWVIEW = set_crosshairs(AVWVIEW);
            
            
        case {'voxel_coordinates'},
            
            
            %         get(AVWVIEW.handles.impos,'Value')
            
            impos = get(AVWVIEW.handles.impos,'string');
            try
                a = eval(impos);
            catch
                try
                    a = eval([' [' impos ']']);
                catch
                    a=get(AVWVIEW.handles.impos,'Value');
                end
            end
            if length(a)==3
                AVWVIEW.slices.sag=round(a(1));
                AVWVIEW.slices.cor=round(a(2));
                AVWVIEW.slices.axi=round(a(3));
                set(AVWVIEW.handles.impos,'string',sprintf(' [%5d, %5d, %5d] ',a));
                set(AVWVIEW.handles.impos,'value',a);
            end
            AVWVIEW = set_coordinates(AVWVIEW);
            %         AVWVIEW = set_display_values(AVWVIEW);
            %         AVWVIEW = set_crosshairs(AVWVIEW);
            AVWVIEW.view = 'axi'; axial_update(AVWVIEW)
            AVWVIEW.view = 'cor'; coronal_update(AVWVIEW)
            AVWVIEW.view = 'sag'; sagittal_update(AVWVIEW)
            
            
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            
        case {'left','right','up','down'},
            
            AVWVIEW = get_slider_position(AVWVIEW);
            
            % what axes are we in?
            if isequal(gca, AVWVIEW.handles.axial_axes),
                switch command,
                    case 'left',
                        % decrease sagittal slice
                        if AVWVIEW.slices.sag > 1,
                            AVWVIEW.slices.sag = AVWVIEW.slices.sag - 1;
                        end
                    case 'right',
                        % increase sagittal slice
                        if AVWVIEW.slices.sag < AVWVIEW.xdim,
                            AVWVIEW.slices.sag = AVWVIEW.slices.sag + 1;
                        end
                    case 'up',
                        % increase coronal slice
                        if AVWVIEW.slices.cor < AVWVIEW.ydim,
                            AVWVIEW.slices.cor = AVWVIEW.slices.cor + 1;
                        end
                    case 'down',
                        % decrease coronal slice
                        if AVWVIEW.slices.cor > 1,
                            AVWVIEW.slices.cor = AVWVIEW.slices.cor - 1;
                        end
                end
                switch command,
                    case {'left','right'}
                        set(AVWVIEW.handles.sagittal_slider,'Value',AVWVIEW.slices.sag);
                        Ssag = squeeze(AVWVIEW.avw.img(AVWVIEW.slices.sag,:,:));
                        set(AVWVIEW.handles.sagittal_image,'CData',Ssag');
                        set(AVWVIEW.handles.sagittal_sliderN,'String',num2str(AVWVIEW.slices.sag));
                        set(AVWVIEW.handles.sagittal_sliderN,'Value',AVWVIEW.slices.sag);
                    case {'up','down'},
                        set(AVWVIEW.handles.coronal_slider,'Value',AVWVIEW.slices.cor);
                        Scor = squeeze(AVWVIEW.avw.img(:,AVWVIEW.slices.cor,:));
                        set(AVWVIEW.handles.coronal_image,'CData',Scor');
                        set(AVWVIEW.handles.coronal_sliderN,'String',num2str(AVWVIEW.slices.cor));
                        set(AVWVIEW.handles.coronal_sliderN,'Value',AVWVIEW.slices.cor);
                end
            end
            
            if isequal(gca, AVWVIEW.handles.coronal_axes),
                switch command,
                    case 'left',
                        % decrease sagittal slice
                        if AVWVIEW.slices.sag > 1,
                            AVWVIEW.slices.sag = AVWVIEW.slices.sag - 1;
                        end
                    case 'right',
                        % increase sagittal slice
                        if AVWVIEW.slices.sag < AVWVIEW.xdim,
                            AVWVIEW.slices.sag = AVWVIEW.slices.sag + 1;
                        end
                    case 'up',
                        % increase axial slice
                        if AVWVIEW.slices.axi < AVWVIEW.zdim,
                            AVWVIEW.slices.axi = AVWVIEW.slices.axi + 1;
                        end
                    case 'down',
                        % decrease axial slice
                        if AVWVIEW.slices.axi > 1,
                            AVWVIEW.slices.axi = AVWVIEW.slices.axi - 1;
                        end
                end
                switch command,
                    case {'left','right'}
                        set(AVWVIEW.handles.sagittal_slider,'Value',AVWVIEW.slices.sag);
                        Ssag = squeeze(AVWVIEW.avw.img(AVWVIEW.slices.sag,:,:));
                        set(AVWVIEW.handles.sagittal_image,'CData',Ssag');
                        set(AVWVIEW.handles.sagittal_sliderN,'String',num2str(AVWVIEW.slices.sag));
                        set(AVWVIEW.handles.sagittal_sliderN,'Value',AVWVIEW.slices.sag);
                    case {'up','down'},
                        set(AVWVIEW.handles.axial_slider,'Value',AVWVIEW.slices.axi);
                        Saxi = squeeze(AVWVIEW.avw.img(:,:,AVWVIEW.slices.axi));
                        set(AVWVIEW.handles.axial_image,'CData',Saxi');
                        set(AVWVIEW.handles.axial_sliderN,'String',num2str(AVWVIEW.slices.axi));
                        set(AVWVIEW.handles.axial_sliderN,'Value',AVWVIEW.slices.axi);
                end
            end
            
            if isequal(gca, AVWVIEW.handles.sagittal_axes),
                switch command,
                    case 'left',
                        % decrease sagittal slice
                        if AVWVIEW.slices.cor > 1,
                            AVWVIEW.slices.cor = AVWVIEW.slices.cor - 1;
                        end
                    case 'right',
                        % increase sagittal slice
                        if AVWVIEW.slices.cor < AVWVIEW.ydim,
                            AVWVIEW.slices.cor = AVWVIEW.slices.cor + 1;
                        end
                    case 'up',
                        % increase axial slice
                        if AVWVIEW.slices.axi < AVWVIEW.zdim,
                            AVWVIEW.slices.axi = AVWVIEW.slices.axi + 1;
                        end
                    case 'down',
                        % decrease axial slice
                        if AVWVIEW.slices.axi > 1,
                            AVWVIEW.slices.axi = AVWVIEW.slices.axi - 1;
                        end
                end
                switch command,
                    case {'left','right'}
                        set(AVWVIEW.handles.coronal_slider,'Value',AVWVIEW.slices.cor);
                        Scor = squeeze(AVWVIEW.avw.img(:,AVWVIEW.slices.cor,:));
                        set(AVWVIEW.handles.coronal_image,'CData',Scor');
                        set(AVWVIEW.handles.coronal_sliderN,'String',num2str(AVWVIEW.slices.cor));
                        set(AVWVIEW.handles.coronal_sliderN,'Value',AVWVIEW.slices.cor);
                    case {'up','down'},
                        set(AVWVIEW.handles.axial_slider,'Value',AVWVIEW.slices.axi);
                        Saxi = squeeze(AVWVIEW.avw.img(:,:,AVWVIEW.slices.axi));
                        set(AVWVIEW.handles.axial_image,'CData',Saxi');
                        set(AVWVIEW.handles.axial_sliderN,'String',num2str(AVWVIEW.slices.axi));
                        set(AVWVIEW.handles.axial_sliderN,'Value',AVWVIEW.slices.axi);
                end
            end
            
            AVWVIEW = set_coordinates(AVWVIEW);
            %AVWVIEW = set_crosshairs(AVWVIEW);
            
            
            
        case {'bgimgfrac'},
            
            bgimgfrac = get(AVWVIEW.handles.bgimgfrac,'string');
            try
                a = eval(bgimgfrac);
            catch
                a=get(AVWVIEW.handles.bgimgfrac,'value')
            end
            AVWVIEW.bgimg_fraction =a;
            axial_update(AVWVIEW);
            coronal_update(AVWVIEW);
            sagittal_update(AVWVIEW);
            
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            
        case {'setclimit'},
            
            switch command,
                case 'setclimit',
                    default =AVWVIEW.default_clim;
                    clim = get(AVWVIEW.handles.clim,'string');
                    try
                        a = eval(clim);
                    catch
                        try
                            a = eval([' [' clim ']']);
                        catch
                            return
                        end
                    end
                    if length(a)==1
                        AVWVIEW.clim(2) = a;
                    else
                        AVWVIEW.clim = a;
                    end
            end
            climString = sprintf(' [%g, %g] ',AVWVIEW.clim);
            set(AVWVIEW.handles.clim,'string',climString);
            
            axial_update(AVWVIEW);
            coronal_update(AVWVIEW);
            sagittal_update(AVWVIEW);
            draw_colorbar(AVWVIEW)
            
            
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            
        case 'setcmap'
            
            cmapIndex = get(AVWVIEW.handles.cmap,'value');
            cmapString = get(AVWVIEW.handles.cmap,'string');
            
            AVWVIEW.cmapString = cmapString{cmapIndex};
            
            if isfield(AVWVIEW.handles,'axial_image'),
                AVWVIEW.cmap = colormap(AVWVIEW.handles.axial_axes,AVWVIEW.cmapString);
            end;
            if isfield(AVWVIEW.handles,'coronal_image'),
                AVWVIEW.cmap = colormap(AVWVIEW.handles.coronal_axes,AVWVIEW.cmapString);
            end;
            if isfield(AVWVIEW.handles,'sagittal_image'),
                AVWVIEW.cmap = colormap(AVWVIEW.handles.sagittal_axes,AVWVIEW.cmapString);
            end;
            AVWVIEW.view = 'axi'; axial_update(AVWVIEW)
            AVWVIEW.view = 'cor'; coronal_update(AVWVIEW)
            AVWVIEW.view = 'sag'; sagittal_update(AVWVIEW)
            draw_colorbar(AVWVIEW)
            
            
            
            
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            
            %     case 'histogram',
            %
            %         avw = AVWVIEW.avw;
            %
            %         % would be nice to use bins that reflect
            %         % the bits per pixel, but it seems to take
            %         % forever for a 16 bit image, otherwise
            %         % use this code:
            %         %     %check the bits per pixel of avw
            %         %     bitpix = avw.hdr.dime.bitpix;
            %         %     % set the bins according to the data type
            %         %     if bitpix <= 8, bins = 0:255; end
            %         %     if bitpix > 8,  bins = 0:65535; end
            %
            %         %bins = linspace(0,intensity_max,255);
            %
            %         [bins,intensity_volume] = avw_histogram(avw);
            %
            %         figure('name','intensity histogram');
            %         bar(bins,intensity_volume);
            %
            %
            
            
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            
        case 'zoom'
            
            x = AVWVIEW.slices.sag;%Petr Krysl
            y = AVWVIEW.slices.cor;
            z = AVWVIEW.slices.axi;
            
            switch AVWVIEW.view,%Petr Krysl
                case 'axi',
                    slice = rot90(AVWVIEW.avw.img(:,:,z));
                case 'cor',
                    slice = rot90(squeeze(AVWVIEW.avw.img(:,y,:)));
                case 'sag',
                    slice = rot90(squeeze(AVWVIEW.avw.img(x,:,:)));
            end
            
            figure('Position',get(0,'ScreenSize'));%Petr Krysl
            %         AVWVIEW.clim
            %         set(gcf,'Toolbar','none');
            im=imagesc(slice,AVWVIEW.clim);
            
            aspect = 1./AVWVIEW.daspect;
            switch AVWVIEW.view,%Petr Krysl
                case 'axi',
                    Color = get(AVWVIEW.handles.axial_image,'CData');
                    asp =aspect([1 2 3]);
                case 'cor',
                    Color = get(AVWVIEW.handles.coronal_image,'CData');
                    asp =aspect([1 3 2]);
                case 'sag',
                    Color = get(AVWVIEW.handles.sagittal_image,'CData');
                    asp =aspect([2 3 1]);
            end
            
            set (im,'CData',Color);
            axis xy off; %zoom on;
            set(gcf,'Color',zeros( 3, 1));
            daspect(asp);
            %         magnifier(gcf);
            
            
            
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            
            
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            
        case 'magnify',
            if (isfield(AVWVIEW,'magnifier')) && (~isempty(AVWVIEW.magnifier))
                cleanup(AVWVIEW.magnifier);
                set(AVWVIEW.gui, ...
                    'WindowButtonDownFcn',  [], ...
                    'WindowButtonUpFcn', [], ...
                    'WindowButtonMotionFcn', [], ...
                    'KeyPressFcn', []);
                AVWVIEW.magnifier = [];
            else
                AVWVIEW.magnifier =magnifier (AVWVIEW.gui);
            end
            
            
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            
            
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            
        case 'quit',
            
            %         flipStatus = get(AVWVIEW.handles.flipStatus,'string');
            %         if strmatch(flipStatus,'L>>R (neurological)'),
            %             fprintf('...returning avw.img to radiological orientation;\n');
            %             fprintf('...use avw_flip to permanently change the orientation.\n');
            %             AVWVIEW.avw.img = flipdim(AVWVIEW.avw.img,1);
            %         end;
            
            % try to assign back into the input struct
            if ~isempty(AVWVIEW.invarname),
                fprintf('...returning data to base workspace struct ''%s''\n',AVWVIEW.invarname);
                assignin('base',AVWVIEW.invarname,AVWVIEW.avw);
            elseif evalin('base','exist(''mri'',''var'')'),
                fprintf('...returning data to base workspace struct ''mri''\n');
                string = ['AVWVIEW = get(', num2str(AVWVIEW.gui),...
                    ',''Userdata''); mri.data = AVWVIEW.avw; clear AVWVIEW;'];
                evalin('base',string);
            else
                fprintf('...returning data into base workspace struct ''avw''\n');
                assignin('base','avw',AVWVIEW.avw);
            end
            close gcbf;
            
        otherwise,
            
    end
    
    
    switch command,
        case 'quit',
        otherwise,
            set(AVWVIEW.gui,'UserData',AVWVIEW);
    end
    
    if nargout > 0,
        varargout{1} = AVWVIEW.avw;
    end
    if nargout > 0,
        varargout{1} = AVWVIEW.avw;
    end
    
    
    return
    
    
    
    
    
    
    
    
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% SUBFUNCTIONS






%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function axial_update(AVWVIEW)
    
    if isfield(AVWVIEW.handles,'axial_image'),
        Saxi = squeeze(AVWVIEW.avw.img(:,:,AVWVIEW.slices.axi));
        color=map2color_method(AVWVIEW.cmap, AVWVIEW.clim, double(Saxi'));
        if ~isempty(AVWVIEW.bgavw)
            bgSaxi =squeeze(AVWVIEW.bgavw.img(:,:,AVWVIEW.slices.axi));
            bgcolor=map2color_method(AVWVIEW.bone, AVWVIEW.bgclim, double(bgSaxi'));
            color =AVWVIEW.bgimg_fraction*bgcolor + (1-AVWVIEW.bgimg_fraction)*color;
        end
        set(AVWVIEW.handles.axial_image,'CData',reshape(color,[size(Saxi'),3]));
    end
    if isfield(AVWVIEW.handles,'axial_sliderN'),
        set(AVWVIEW.handles.axial_sliderN,'String',num2str(AVWVIEW.slices.axi));
        set(AVWVIEW.handles.axial_sliderN,'Value',AVWVIEW.slices.axi);
    end
    if isfield(AVWVIEW.handles,'axial_slider'),
        set(AVWVIEW.handles.axial_slider,'Value',AVWVIEW.slices.axi);
    end;
    
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function coronal_update(AVWVIEW)
    
    if isfield(AVWVIEW.handles,'coronal_image'),
        Scor = squeeze(AVWVIEW.avw.img(:,AVWVIEW.slices.cor,:));
        color=map2color_method(AVWVIEW.cmap, AVWVIEW.clim, double(Scor'));
        if ~isempty(AVWVIEW.bgavw)
            bgScor =squeeze(AVWVIEW.bgavw.img(:,AVWVIEW.slices.cor,:));
            bgcolor=map2color_method(AVWVIEW.bone, AVWVIEW.bgclim, double(bgScor'));
            color =AVWVIEW.bgimg_fraction*bgcolor + (1-AVWVIEW.bgimg_fraction)*color;
        end
        set(AVWVIEW.handles.coronal_image,'CData',reshape(color,[size(Scor'),3]));
    end
    if isfield(AVWVIEW.handles,'coronal_sliderN'),
        set(AVWVIEW.handles.coronal_sliderN,'String',num2str(AVWVIEW.slices.cor));
        set(AVWVIEW.handles.coronal_sliderN,'Value',AVWVIEW.slices.cor);
    end
    if isfield(AVWVIEW.handles,'coronal_slider'),
        set(AVWVIEW.handles.coronal_slider,'Value',AVWVIEW.slices.cor);
    end;
    
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function sagittal_update(AVWVIEW)
    
    if isfield(AVWVIEW.handles,'sagittal_image'),
        Ssag = squeeze(AVWVIEW.avw.img(AVWVIEW.slices.sag,:,:))';% transpose?
        color=map2color_method(AVWVIEW.cmap, AVWVIEW.clim, double(Ssag));
        if ~isempty(AVWVIEW.bgavw)
            bgSsag =squeeze(AVWVIEW.bgavw.img(AVWVIEW.slices.sag,:,:))';% transpose?
            bgcolor=map2color_method(AVWVIEW.bone, AVWVIEW.bgclim, double(bgSsag));
            color =AVWVIEW.bgimg_fraction*bgcolor + (1-AVWVIEW.bgimg_fraction)*color;
        end
        set(AVWVIEW.handles.sagittal_image,'CData',reshape(color,[size(Ssag),3]));
    end
    if isfield(AVWVIEW.handles,'sagittal_sliderN'),
        set(AVWVIEW.handles.sagittal_sliderN,'String',num2str(AVWVIEW.slices.sag));
        set(AVWVIEW.handles.sagittal_sliderN,'Value',AVWVIEW.slices.sag);
    end
    if isfield(AVWVIEW.handles,'sagittal_slider'),
        set(AVWVIEW.handles.sagittal_slider,'Value',AVWVIEW.slices.sag);
    end;
    
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function AVWVIEW = set_crosshairs(AVWVIEW)
    
    current_axes = gca;
    
    [AVWVIEW, metric] = slices2metric(AVWVIEW);
    
    if isfield(AVWVIEW.handles,'axial_axes'),
        
        axes(AVWVIEW.handles.axial_axes)
        
        axialXline = findobj('Tag','axialXline');
        if axialXline,
            delete(axialXline);
        end
        axialYline = findobj('Tag','axialYline');
        if axialYline,
            delete(axialYline);
        end
        
        AVWVIEW.handles.axial_xlim = get(AVWVIEW.handles.axial_axes,'Xlim');
        AVWVIEW.handles.axial_ylim = get(AVWVIEW.handles.axial_axes,'Ylim');
        
        % Petr Krysl: added ,'HitTest','off' because a mouse click
        % would be swallowed by a line that was clicked on
        AVWVIEW.handles.axial_xline = line('Xdata',[metric.sag metric.sag]-0.5,'Ydata',AVWVIEW.handles.axial_ylim,'HitTest','off');
        AVWVIEW.handles.axial_yline = line('Ydata',[metric.cor metric.cor]-0.5,'Xdata',AVWVIEW.handles.axial_xlim,'HitTest','off');
        
        set(AVWVIEW.handles.axial_xline,'Color','b','EraseMode','xor','Tag','axialXline');
        set(AVWVIEW.handles.axial_yline,'Color','b','EraseMode','xor','Tag','axialYline');
        
        if get(AVWVIEW.handles.crosshairs,'value'),
            set(AVWVIEW.handles.axial_xline,'visible','on');
            set(AVWVIEW.handles.axial_yline,'visible','on');
        else
            set(AVWVIEW.handles.axial_xline,'visible','off');
            set(AVWVIEW.handles.axial_yline,'visible','off');
        end
    end
    
    if isfield(AVWVIEW.handles,'coronal_axes'),
        
        axes(AVWVIEW.handles.coronal_axes);
        
        coronalXline = findobj('Tag','coronalXline');
        if coronalXline,
            delete(coronalXline);
        end
        coronalYline = findobj('Tag','coronalYline');
        if coronalYline,
            delete(coronalYline);
        end
        
        AVWVIEW.handles.coronal_xlim = get(AVWVIEW.handles.coronal_axes,'Xlim');
        AVWVIEW.handles.coronal_ylim = get(AVWVIEW.handles.coronal_axes,'Ylim');
        AVWVIEW.handles.coronal_xline = line('Xdata',[metric.sag metric.sag]-0.5,'Ydata',AVWVIEW.handles.coronal_ylim,'HitTest','off');
        AVWVIEW.handles.coronal_yline = line('Ydata',[metric.axi metric.axi]-0.5,'Xdata',AVWVIEW.handles.coronal_xlim,'HitTest','off');
        set(AVWVIEW.handles.coronal_xline,'Color','b','EraseMode','xor','Tag','coronalXline');
        set(AVWVIEW.handles.coronal_yline,'Color','b','EraseMode','xor','Tag','coronalYline');
        
        if get(AVWVIEW.handles.crosshairs,'value'),
            set(AVWVIEW.handles.coronal_xline,'visible','on');
            set(AVWVIEW.handles.coronal_yline,'visible','on');
        else
            set(AVWVIEW.handles.coronal_xline,'visible','off');
            set(AVWVIEW.handles.coronal_yline,'visible','off');
        end
    end
    
    if isfield(AVWVIEW.handles,'sagittal_axes'),
        
        axes(AVWVIEW.handles.sagittal_axes);
        
        sagittalXline = findobj('Tag','sagittalXline');
        if sagittalXline,
            delete(sagittalXline);
        end
        sagittalYline = findobj('Tag','sagittalYline');
        if sagittalYline,
            delete(sagittalYline);
        end
        
        AVWVIEW.handles.sagittal_xlim = get(AVWVIEW.handles.sagittal_axes,'Xlim');
        AVWVIEW.handles.sagittal_ylim = get(AVWVIEW.handles.sagittal_axes,'Ylim');
        AVWVIEW.handles.sagittal_xline = line('Xdata',[metric.cor metric.cor]-0.5,'Ydata',AVWVIEW.handles.sagittal_ylim,'HitTest','off');
        AVWVIEW.handles.sagittal_yline = line('Ydata',[metric.axi metric.axi]-0.5,'Xdata',AVWVIEW.handles.sagittal_xlim,'HitTest','off');
        set(AVWVIEW.handles.sagittal_xline,'Color','b','EraseMode','xor','Tag','sagittalXline');
        set(AVWVIEW.handles.sagittal_yline,'Color','b','EraseMode','xor','Tag','sagittalYline');
        
        if get(AVWVIEW.handles.crosshairs,'value'),
            set(AVWVIEW.handles.sagittal_xline,'visible','on');
            set(AVWVIEW.handles.sagittal_yline,'visible','on');
        else
            set(AVWVIEW.handles.sagittal_xline,'visible','off');
            set(AVWVIEW.handles.sagittal_yline,'visible','off');
        end
        
    end
    
    if (ishandle(current_axes))
        axes(current_axes);% conflict with magnifier?
    end
    
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function AVWVIEW = set_coordinates(AVWVIEW)
    
    % set the axis coordinates to voxels, mm or meters
    
    s = size(AVWVIEW.avw.img);
    if length(s) > 0, xdim = s(1); else xdim = 1; end
    if length(s) > 1, ydim = s(2); else ydim = 1; end
    if length(s) > 2, zdim = s(3); else zdim = 1; end
    
    % initialise for voxel coordinates
    xpixdim = double(AVWVIEW.avw.hdr.dime.pixdim(2));
    ypixdim = double(AVWVIEW.avw.hdr.dime.pixdim(3));
    zpixdim = double(AVWVIEW.avw.hdr.dime.pixdim(4));
    xdata = [0 xdim];
    ydata = [0 ydim];
    zdata = [0 zdim];
    
    aspect = 1./AVWVIEW.daspect; %AVWVIEW.avw.hdr.dime.pixdim(2:4);
    
    
    if get(AVWVIEW.handles.coord,'value') == 2,    % mm
        xdata = xdata .* xpixdim;
        ydata = ydata .* ypixdim;
        zdata = zdata .* zpixdim;
        aspect = [1 1 1];
    end
    
    if get(AVWVIEW.handles.coord,'value') == 3,    % meters
        xpixdim = xpixdim / 1000;
        ypixdim = ypixdim / 1000;
        zpixdim = zpixdim / 1000;
        xdata = xdata .* xpixdim;
        ydata = ydata .* ypixdim;
        zdata = zdata .* zpixdim;
        aspect = [1 1 1];
    end
    
    
    if isfield(AVWVIEW.handles,'axial_image'),
        set(AVWVIEW.handles.axial_axes,'Xlim',xdata);
        set(AVWVIEW.handles.axial_axes,'Ylim',ydata);
        set(AVWVIEW.handles.axial_axes,'Zlim',zdata);
        set(AVWVIEW.handles.axial_image,'Xdata',xdata);
        set(AVWVIEW.handles.axial_image,'Ydata',ydata);
        daspect(AVWVIEW.handles.axial_axes,aspect([1 2 3]));
    end;
    if isfield(AVWVIEW.handles,'coronal_image'),
        set(AVWVIEW.handles.coronal_axes,'Xlim',xdata);
        set(AVWVIEW.handles.coronal_axes,'Ylim',zdata);
        set(AVWVIEW.handles.coronal_axes,'Zlim',ydata);
        set(AVWVIEW.handles.coronal_image,'Xdata',xdata);
        set(AVWVIEW.handles.coronal_image,'Ydata',zdata);
        daspect(AVWVIEW.handles.coronal_axes,aspect([1 3 2]));
    end;
    if isfield(AVWVIEW.handles,'sagittal_image'),
        set(AVWVIEW.handles.sagittal_axes,'Xlim',ydata);
        set(AVWVIEW.handles.sagittal_axes,'Ylim',zdata);
        set(AVWVIEW.handles.sagittal_axes,'Zlim',xdata);
        set(AVWVIEW.handles.sagittal_image,'Xdata',ydata);
        set(AVWVIEW.handles.sagittal_image,'Ydata',zdata);
        daspect(AVWVIEW.handles.sagittal_axes,aspect([2 3 1]));
    end;
    
    AVWVIEW = set_display_values(AVWVIEW);
    AVWVIEW = set_crosshairs(AVWVIEW);
    
end




%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function AVWVIEW = get_current_position(AVWVIEW),
    
    AVWVIEW = get_slider_position(AVWVIEW);
    [AVWVIEW, metric] = slices2metric(AVWVIEW);
    
    switch AVWVIEW.view,
        case 'sag',
            currentpoint = get(get(AVWVIEW.handles.sagittal_image,'Parent'),'CurrentPoint');
            metric.cor = currentpoint(1,1);
            metric.axi = currentpoint(1,2);
        case 'cor',
            currentpoint = get(get(AVWVIEW.handles.coronal_image,'Parent'),'CurrentPoint');
            metric.sag = currentpoint(2,1);
            metric.axi = currentpoint(2,2);
        case 'axi',
            currentpoint = get(get(AVWVIEW.handles.axial_image,'Parent'),'CurrentPoint');
            metric.sag = currentpoint(2,1);
            metric.cor = currentpoint(2,2);
    end
    
    AVWVIEW = metric2slices(AVWVIEW,metric);
    AVWVIEW = check_slices(AVWVIEW);
    
end




%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function AVWVIEW = get_slider_position(AVWVIEW),
    
    [AVWVIEW.slices.sag,AVWVIEW.slices.cor,AVWVIEW.slices.axi] = deal(0);
    
    if isfield(AVWVIEW.handles,'sagittal_slider'),
        if ishandle(AVWVIEW.handles.sagittal_slider),
            AVWVIEW.slices.sag = round(get(AVWVIEW.handles.sagittal_slider,'Value'));
        end
    end
    if AVWVIEW.slices.sag == 0,
        if isfield(AVWVIEW.handles,'sagittal_sliderN'),
            if ishandle(AVWVIEW.handles.sagittal_sliderN),
                AVWVIEW.slices.sag = round(get(AVWVIEW.handles.sagittal_sliderN,'Value'));
            end
        end
    end
    
    if isfield(AVWVIEW.handles,'coronal_slider'),
        if ishandle(AVWVIEW.handles.coronal_slider),
            AVWVIEW.slices.cor = round(get(AVWVIEW.handles.coronal_slider,'Value'));
        end
    end
    if AVWVIEW.slices.cor == 0,
        if isfield(AVWVIEW.handles,'coronal_sliderN'),
            if ishandle(AVWVIEW.handles.coronal_sliderN),
                AVWVIEW.slices.cor = round(get(AVWVIEW.handles.coronal_sliderN,'Value'));
            end
        end
    end
    
    if isfield(AVWVIEW.handles,'axial_slider'),
        if ishandle(AVWVIEW.handles.axial_slider),
            AVWVIEW.slices.axi = round(get(AVWVIEW.handles.axial_slider,'Value'));
        end
    end
    if AVWVIEW.slices.axi == 0,
        if isfield(AVWVIEW.handles,'axial_sliderN'),
            if ishandle(AVWVIEW.handles.axial_sliderN),
                AVWVIEW.slices.axi = round(get(AVWVIEW.handles.axial_sliderN,'Value'));
            end
        end
    end
    
    AVWVIEW = check_slices(AVWVIEW);
    
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function AVWVIEW = check_slices(AVWVIEW),
    
    adjust = 0;
    
    [ SagSize, CorSize, AxiSize ] = size(AVWVIEW.avw.img);
    
    if AVWVIEW.slices.sag > SagSize,
        AVWVIEW.slices.sag = SagSize;
        adjust = 1;
    end;
    if AVWVIEW.slices.sag < 1,
        AVWVIEW.slices.sag = 1;
        adjust = 1;
    end;
    if AVWVIEW.slices.cor > CorSize,
        AVWVIEW.slices.cor = CorSize;
        adjust = 1;
    end;
    if AVWVIEW.slices.cor < 1,
        AVWVIEW.slices.cor = 1;
        adjust = 1;
    end;
    if AVWVIEW.slices.axi > AxiSize,
        AVWVIEW.slices.axi = AxiSize;
        adjust = 1;
    end;
    if AVWVIEW.slices.axi < 1,
        AVWVIEW.slices.axi = 1;
        adjust = 1;
    end;
    
    if adjust,
        AVWVIEW = slices2metric(AVWVIEW);
    end
    
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function AVWVIEW = set_display_values(AVWVIEW),
    
    % get coordinates of selected voxel and the image intensity there
    
    sag = AVWVIEW.slices.sag;
    cor = AVWVIEW.slices.cor;
    axi = AVWVIEW.slices.axi;
    
    imgvalue = AVWVIEW.avw.img(sag,cor,axi);
    
    set(AVWVIEW.handles.imval,'String',sprintf('%g ',imgvalue));
    set(AVWVIEW.handles.imval,'Value',imgvalue);
    % PK: set scaled value
    cal_Span=double(AVWVIEW.avw.hdr.dime.cal_max)-double(AVWVIEW.avw.hdr.dime.cal_min);
    imfraction = (double(imgvalue)-double(intmin('int16')))/(double(intmax('int16'))-double(intmin('int16')));
    imgvalue =double(AVWVIEW.avw.hdr.dime.cal_min)  +cal_Span*imfraction;
    set(AVWVIEW.handles.imvalscaled,'String',sprintf('%g',imgvalue));
    set(AVWVIEW.handles.imval,'Value',imgvalue);
    
    % Now update the image position text for the selected voxel
    
    [AVWVIEW, metric] = slices2metric(AVWVIEW);
    sag = metric.sag;
    cor = metric.cor;
    axi = metric.axi;
    
    if get(AVWVIEW.handles.coord,'value') == 2,    % mm
        string = sprintf('%7.3f, %7.3f, %7.3f ',sag,cor,axi);%Petr Krysl
    elseif get(AVWVIEW.handles.coord,'value') == 3,    % meters
        string = sprintf('%7.3f, %7.3f, %7.3f ',sag,cor,axi);%Petr Krysl
    else
        string = sprintf('%5d, %5d, %5d ',sag,cor,axi);%Petr Krysl
    end
    
    
    set(AVWVIEW.handles.impos,'String',string);
    set(AVWVIEW.handles.impos,'Value',[sag,cor,axi]);
    
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [AVWVIEW,metric] = slices2metric(AVWVIEW),
    
    AVWVIEW.imgXYZ.voxels = [AVWVIEW.slices.sag,AVWVIEW.slices.cor,AVWVIEW.slices.axi];
    AVWVIEW.imgXYZ.meters = double(AVWVIEW.imgXYZ.voxels) .* AVWVIEW.scale2meters;
    AVWVIEW.imgXYZ.mm     = double(AVWVIEW.imgXYZ.voxels) .* AVWVIEW.scale2mm;
    
    coord_value = get(AVWVIEW.handles.coord,'value');
    
    if coord_value == 2,
        % using mm
        img_mm = AVWVIEW.imgXYZ.mm;
        metric.axi = img_mm(3);
        metric.cor = img_mm(2);
        metric.sag = img_mm(1);
    elseif coord_value == 3,
        % using meters
        img_meters = AVWVIEW.imgXYZ.meters;
        metric.axi = img_meters(3);
        metric.cor = img_meters(2);
        metric.sag = img_meters(1);
    else
        % voxels
        metric = AVWVIEW.slices;
    end;
    
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function AVWVIEW = metric2slices(AVWVIEW,metric),
    
    coord_value = get(AVWVIEW.handles.coord,'value');
    
    if coord_value == 2,
        % using mm
        xpix = double(AVWVIEW.avw.hdr.dime.pixdim(2));
        ypix = double(AVWVIEW.avw.hdr.dime.pixdim(3));
        zpix = double(AVWVIEW.avw.hdr.dime.pixdim(4));
        AVWVIEW.slices.axi = round(metric.axi / zpix);
        AVWVIEW.slices.cor = round(metric.cor / ypix);
        AVWVIEW.slices.sag = round(metric.sag / xpix);
    elseif coord_value == 3,
        % using meters
        xpix = double(AVWVIEW.avw.hdr.dime.pixdim(2)) / 1000;
        ypix = double(AVWVIEW.avw.hdr.dime.pixdim(3)) / 1000;
        zpix = double(AVWVIEW.avw.hdr.dime.pixdim(4)) / 1000;
        AVWVIEW.slices.axi = round(metric.axi / zpix);
        AVWVIEW.slices.cor = round(metric.cor / ypix);
        AVWVIEW.slices.sag = round(metric.sag / xpix);
    else
        % voxels
        AVWVIEW.slices.axi = round(metric.axi);
        AVWVIEW.slices.cor = round(metric.cor);
        AVWVIEW.slices.sag = round(metric.sag);
    end;
    
end





%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function AVWVIEW = init(avw,options),
    
    % try to keep track of the input struct
    % Petr Krysl
    % if exist('invarname','var'),
    %     if ~isempty(invarname),
    %         AVWVIEW.invarname = invarname;
    %     else
    %         AVWVIEW.invarname = '';
    %     end
    % else
    %     AVWVIEW.invarname = '';
    % end
    if isfield(options,'name')
        if ~isempty(options.name),
            AVWVIEW.invarname = options.name;
        else
            AVWVIEW.invarname = '';
        end
    else
        AVWVIEW.invarname = '';
    end
    
    AVWVIEW.bgimg_fraction =0.45;
    if isfield(options,'bgimg_fraction')
        if ~isempty(options.bgimg_fraction),
            AVWVIEW.bgimg_fraction = options.bgimg_fraction;
        end
    end
    AVWVIEW.bone = [0         0    0.0052
        0.0139    0.0139    0.0243
        0.0278    0.0278    0.0434
        0.0417    0.0417    0.0625
        0.0556    0.0556    0.0816
        0.0694    0.0694    0.1007
        0.0833    0.0833    0.1198
        0.0972    0.0972    0.1389
        0.1111    0.1111    0.1580
        0.1250    0.1250    0.1771
        0.1389    0.1389    0.1962
        0.1528    0.1528    0.2153
        0.1667    0.1667    0.2344
        0.1806    0.1806    0.2535
        0.1944    0.1944    0.2726
        0.2083    0.2083    0.2917
        0.2222    0.2222    0.3108
        0.2361    0.2361    0.3299
        0.2500    0.2500    0.3490
        0.2639    0.2639    0.3681
        0.2778    0.2778    0.3872
        0.2917    0.2917    0.4062
        0.3056    0.3056    0.4253
        0.3194    0.3194    0.4444
        0.3333    0.3385    0.4583
        0.3472    0.3576    0.4722
        0.3611    0.3767    0.4861
        0.3750    0.3958    0.5000
        0.3889    0.4149    0.5139
        0.4028    0.4340    0.5278
        0.4167    0.4531    0.5417
        0.4306    0.4722    0.5556
        0.4444    0.4913    0.5694
        0.4583    0.5104    0.5833
        0.4722    0.5295    0.5972
        0.4861    0.5486    0.6111
        0.5000    0.5677    0.6250
        0.5139    0.5868    0.6389
        0.5278    0.6059    0.6528
        0.5417    0.6250    0.6667
        0.5556    0.6441    0.6806
        0.5694    0.6632    0.6944
        0.5833    0.6823    0.7083
        0.5972    0.7014    0.7222
        0.6111    0.7205    0.7361
        0.6250    0.7396    0.7500
        0.6389    0.7587    0.7639
        0.6528    0.7778    0.7778
        0.6745    0.7917    0.7917
        0.6962    0.8056    0.8056
        0.7179    0.8194    0.8194
        0.7396    0.8333    0.8333
        0.7613    0.8472    0.8472
        0.7830    0.8611    0.8611
        0.8047    0.8750    0.8750
        0.8264    0.8889    0.8889
        0.8481    0.9028    0.9028
        0.8698    0.9167    0.9167
        0.8915    0.9306    0.9306
        0.9132    0.9444    0.9444
        0.9349    0.9583    0.9583
        0.9566    0.9722    0.9722
        0.9783    0.9861    0.9861
        1.0000    1.0000    1.0000];
    AVWVIEW.bgavw = [];
    if isfield(options,'bgavw')
        if (~isempty(options.bgavw))
            if (options.bgavw.hdr.dime.dim(2:4) == avw.hdr.dime.dim(2:4))
                AVWVIEW.bgavw = options.bgavw;
                intensityMax = max(max(max(AVWVIEW.bgavw.img)));
                intensityMin = min(min(min(AVWVIEW.bgavw.img)));
                AVWVIEW.bgclim = [intensityMin intensityMax];
            end
        end
    else
        AVWVIEW.bgclim = [ 0 1];
    end
    
    % GUI General Parameters
    GUIwidth  = 150;
    GUIheight = 50;
    
    version = '[$Revision: 2.0 $]';
    name = sprintf('AVW View [v%s]',version(12:16));
    
    if isfield(avw,'fileprefix'),
        if ~isempty(avw.fileprefix),
            [pathstr,filename,ext] = fileparts(avw.fileprefix);%Petr Krysl
            format = strcat('%+',sprintf('%d',length(filename)+1),'s');
            name = [name,' - ',filename,ext];%Petr Krysl
        end
    end
    
    % The Zbuffer provides smooth slice animations (OpenGL is no good)
    GUI = figure('Name',name,'Tag','AVWVIEW','units','characters',...
        'BackingStore','off',...
        'NumberTitle','off','color',0.9+[0 0 0],...
        'MenuBar','figure','Position',[1 1 GUIwidth GUIheight],...
        'Renderer','zbuffer');
    
    movegui(GUI,'center');
    
    set(GUI,'KeyPressFcn','avw_view([],[],''keypress'');');
    
    AVWVIEW.gui = GUI;
    figure (GUI)%Petr Krysl
    
    Font.FontName   = 'fixed';
    Font.FontUnits  = 'Pixels';
    Font.FontSize   = 12;
    Font.FontWeight = 'normal';
    Font.FontAngle  = 'normal';
    
    
    AVWVIEW.shading = 'flat';
    shading(AVWVIEW.shading)
    
    
    
    % 	% determine the datatype of avw.img
    % 	switch double(avw.hdr.dime.bitpix),
    % 	case 1,
    %         fprintf('...converting avw.img to uint8 for viewing only.\n\n');
    %         avw.img = uint8(avw.img);
    % 	case 8,
    %         fprintf('...converting avw.img to uint8 for viewing only.\n\n');
    %         avw.img = uint8(avw.img);
    % 	case 16,
    %         fprintf('...converting avw.img to uint16 for viewing only.\n\n');
    %         avw.img = uint16(avw.img);
    % 	case {32,64},
    %         % make sure it is double, not single
    %         avw.img = double(avw.img);
    % 	otherwise,
    %         % do nothing, leave it as is
    % 	end
    
    
    % calculate image stats
    intensityMean = mean(mean(mean(avw.img)));
    % The following two operations are too inefficient with respect
    % to memory use: "out of memory" error
    % intensityMeanRobust = mean(mean(mean(avw.img(find(avw.img)))));%Petr Krysl
    intensityMeanRobust = mean(mean(mean(avw.img)));
    % intensityStdev = std(std(std(avw.img)));%Petr Krysl
    intensityStdev = 0;
    
    intensityMax = max(max(max(avw.img)));
    intensityMin = min(min(min(avw.img)));
    
    % try to adjust for large intensity extremes
    if intensityMax > 255,
        % got 16 bit data, at least
        adjust = 0.5;
    else
        adjust = 0.9;
    end
    %AVWVIEW.clim = [0 (intensityMeanRobust + (10 * intensityStdev)) ];
    % AVWVIEW.clim = [0 (intensityMax * adjust)];%Petr Krysl
    AVWVIEW.default_clim = [intensityMin intensityMax];
    if isfield(options,'clim')
        AVWVIEW.clim = options.clim;
    else
        AVWVIEW.clim = AVWVIEW.default_clim;
    end
    if isfield(options,'pois')
        AVWVIEW.pois = options.pois;
    else
        AVWVIEW.pois = [];
    end
    
    
    
    AVWVIEW.xdim = size(avw.img,1);
    AVWVIEW.ydim = size(avw.img,2);
    AVWVIEW.zdim = size(avw.img,3);
    
    AVWVIEW.slices.sag = 1;
    AVWVIEW.slices.cor = 1;
    AVWVIEW.slices.axi = 1;
    if AVWVIEW.xdim > 1, AVWVIEW.slices.sag = floor(AVWVIEW.xdim/2); end
    if AVWVIEW.ydim > 1, AVWVIEW.slices.cor = floor(AVWVIEW.ydim/2); end
    if AVWVIEW.zdim > 1, AVWVIEW.slices.axi = floor(AVWVIEW.zdim/2); end
    
    % store the volume center for later reference when
    % calculating fiducial locations
    center = avw_center(avw);
    AVWVIEW.center = center.abs.voxels;
    
    % set the default origin at the center
    AVWVIEW.origin  = AVWVIEW.center;
    
    AVWVIEW.pixdim = double(avw.hdr.dime.pixdim(2:4));
    
    AVWVIEW.scale2mm     = AVWVIEW.pixdim;          % vol scale in mm
    AVWVIEW.scale2meters = AVWVIEW.pixdim ./ 1000;  % vol scale in meters
    AVWVIEW.daspect      = AVWVIEW.pixdim ./ min(AVWVIEW.pixdim);
    
    xPlotSize = 0.41;
    yPlotSize = 0.41;
    
    if isfield(options,'cmap')
        AVWVIEW.cmapString = options.cmap;
    else
        AVWVIEW.cmapString = 'gray';
    end
    
    AVWVIEW.cmap = colormap(AVWVIEW.cmapString);
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % figure (GUI)%Petr Krysl
    
    % Axial Slice
    if AVWVIEW.xdim > 1 & AVWVIEW.ydim > 1,
        
        maxdim=max( [AVWVIEW.xdim,AVWVIEW.ydim,AVWVIEW.zdim].*AVWVIEW.pixdim);
        fx=(AVWVIEW.xdim*AVWVIEW.pixdim(1)/maxdim);
        fy=(AVWVIEW.ydim*AVWVIEW.pixdim(2)/maxdim);
        handles.axial_subplot = subplot('position',[0.07 0.07 xPlotSize*fx yPlotSize*fy]);
        
        Saxial = squeeze(avw.img(:,:,AVWVIEW.slices.axi));
        
        %handles.axial_image = pcolor(double(Saxial)); colormap(gray); shading interp
        %pcolor(Saxial'); colormap(gray); shading flat
        
        %surf(avw.img(:,:,20)','edgecolor','none'); view(2); axis tight; colormap(flipdim(gray,1)); colorbar
        %surf(avw.img(:,:,20)','edgecolor','none'); view(2); axis tight; colormap(gray)
        %image(avw.img(:,:,20)'); axis image; colormap(gray)
        %imagesc(avw.img(:,:,20)'); axis image; colormap(gray)
        
        %handles.axial_image = image('Cdata',Saxial','CDataMapping','scaled',...
        %    'XData',[0 xdim],'YData',[0 ydim]);
        
        if AVWVIEW.clim(2) <= AVWVIEW.clim(1)%Petr Krysl
            AVWVIEW.clim= [0, 1 ];
        end
        handles.axial_image = imagesc([0,AVWVIEW.xdim],[0,AVWVIEW.ydim],Saxial',AVWVIEW.clim);
        
        handles.axial_axes = gca;
        set(gca,'YDir','normal','XLimMode','manual','YLimMode','manual',...
            'ClimMode','manual','YColor',[0, 0, 0],'XColor',[0, 0, 0])
        
        title('Axial','Color',[0, 0, 0])
        ylabel('Y')
        xlabel('X')
        %xlabel('(Right <<) X (>> Left)'); % default radiological orientation for Analyze
        %xlabel('(Left <<) X (>> Right)')
        
        % This callback navigates with mouse button click
        set(handles.axial_image,'ButtonDownFcn','avw_view([],[],''axial_image'');');
        
        GUIheight = 0.38;
        
        if AVWVIEW.zdim > 1,
            slider_step(1) = 1/(AVWVIEW.zdim);
            slider_step(2) = 1/(AVWVIEW.zdim);
            handles.axial_slider = uicontrol('Parent',GUI,'Style','slider',...
                'Units','Normalized', Font,...
                'Position',[.55+0.05 GUIheight .113 0.0313], 'HorizontalAlignment', 'center',...
                'BusyAction','cancel',...
                'TooltipString','Axial slice navigation',...
                'Min',1,'Max',AVWVIEW.zdim,'SliderStep',slider_step,'Value',AVWVIEW.slices.axi,...
                'Callback','avw_view([],[],''axial_slider'');');
        end
        handles.axial_sliderN = uicontrol('Parent',GUI,'Style','text',...
            'Units','Normalized', Font, ...
            'Position',[.67+0.05 GUIheight .045 0.0313], 'HorizontalAlignment', 'center',...
            'BusyAction','cancel',...
            'TooltipString','Axial slice number',...
            'String',num2str(AVWVIEW.slices.axi),'Value',AVWVIEW.slices.axi);
        handles.axial_sliderT = uicontrol('Parent',GUI,'Style','text',...
            'Units','Normalized', Font, ...
            'Position',[.72+0.05 GUIheight .08 0.0313], 'HorizontalAlignment', 'left',...
            'BusyAction','cancel',...
            'TooltipString','Axial slice navigation',...
            'String','Axial');
        
        handles.axial_xlim = get(handles.axial_axes,'Xlim');
        handles.axial_ylim = get(handles.axial_axes,'Ylim');
        handles.axial_xline = line('Xdata',[AVWVIEW.slices.sag AVWVIEW.slices.sag],'Ydata',handles.axial_ylim);
        handles.axial_yline = line('Ydata',[AVWVIEW.slices.cor AVWVIEW.slices.cor],'Xdata',handles.axial_xlim);
        set(handles.axial_xline,'Color','b','EraseMode','xor','Tag','axialXline');
        set(handles.axial_yline,'Color','b','EraseMode','xor','Tag','axialYline');
        
    end
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    % Coronal Slice
    if AVWVIEW.xdim > 1 & AVWVIEW.zdim > 1,
        
        maxdim=max( [AVWVIEW.xdim,AVWVIEW.ydim,AVWVIEW.zdim].*AVWVIEW.pixdim);
        fx=(AVWVIEW.xdim*AVWVIEW.pixdim(1)/maxdim);
        fz=(AVWVIEW.zdim*AVWVIEW.pixdim(3)/maxdim);
        handles.coronal_subplot = subplot('position',[0.07 0.55 xPlotSize*fx yPlotSize*fz]);
        
        Scor = squeeze(avw.img(:,AVWVIEW.slices.cor,:));
        handles.coronal_image = imagesc([0,AVWVIEW.xdim],[0,AVWVIEW.zdim],Scor',AVWVIEW.clim);
        
        handles.coronal_axes = gca;
        set(gca,'YDir','normal','XLimMode','manual','YLimMode','manual',...
            'ClimMode','manual','YColor',[0, 0, 0],'XColor',[0, 0, 0])
        
        %xlabel('(Left <<) X (>> Right)')
        %xlabel('(Right <<) X (>> Left)')
        xlabel('X')
        ylabel('Z')
        title('Coronal','Color',[0, 0, 0])
        
        % This callback navigates with left click
        set(handles.coronal_image,'ButtonDownFcn','avw_view([],[],''coronal_image'');');
        
        GUIheight = 0.42;
        
        if AVWVIEW.ydim > 1,
            slider_step(1) = 1/(AVWVIEW.ydim);
            slider_step(2) = 1/(AVWVIEW.ydim);
            handles.coronal_slider = uicontrol('Parent',GUI,'Style','slider',...
                'Units','Normalized', Font, ...
                'Position',[.55+0.05 GUIheight .113 0.0313], 'HorizontalAlignment', 'center',...
                'BusyAction','cancel',...
                'TooltipString','Coronal slice navigation',...
                'Min',1,'Max',AVWVIEW.ydim,'SliderStep',slider_step,'Value',AVWVIEW.slices.cor,...
                'Callback','avw_view([],[],''coronal_slider'');');
        end
        handles.coronal_sliderN = uicontrol('Parent',GUI,'Style','text',...
            'Units','Normalized', Font, ...
            'Position',[.67+0.05 GUIheight .045 0.0313], 'HorizontalAlignment', 'center',...
            'BusyAction','cancel',...
            'TooltipString','Coronal slice number',...
            'String',num2str(AVWVIEW.slices.cor),'Value',AVWVIEW.slices.cor);
        handles.coronal_sliderT = uicontrol('Parent',GUI,'Style','text',...
            'Units','Normalized', Font, ...
            'Position',[.72+0.05 GUIheight .08 0.0313], 'HorizontalAlignment', 'left',...
            'BusyAction','cancel',...
            'TooltipString','Coronal slice navigation',...
            'String','Coronal');
        
        handles.coronal_xlim = get(handles.coronal_axes,'Xlim');
        handles.coronal_ylim = get(handles.coronal_axes,'Ylim');
        handles.coronal_xline = line('Xdata',[AVWVIEW.slices.sag AVWVIEW.slices.sag],'Ydata',handles.coronal_ylim);
        handles.coronal_yline = line('Ydata',[AVWVIEW.slices.axi AVWVIEW.slices.axi],'Xdata',handles.coronal_xlim);
        set(handles.coronal_xline,'Color','b','EraseMode','xor','Tag','coronalXline');
        set(handles.coronal_yline,'Color','b','EraseMode','xor','Tag','coronalYline');
        
    end
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    % Sagittal Slice
    if AVWVIEW.ydim > 1 & AVWVIEW.zdim > 1,
        
        maxdim=max( [AVWVIEW.xdim,AVWVIEW.ydim,AVWVIEW.zdim].*AVWVIEW.pixdim);
        fy=(AVWVIEW.ydim*AVWVIEW.pixdim(2)/maxdim);
        fz=(AVWVIEW.zdim*AVWVIEW.pixdim(3)/maxdim);
        handles.sagittal_subplot = subplot('position',[0.55 0.55 xPlotSize*fy yPlotSize*fz]);
        
        Ssag = squeeze(avw.img(AVWVIEW.slices.sag,:,:));
        handles.sagittal_image = imagesc([0,AVWVIEW.ydim],[0,AVWVIEW.zdim],Ssag',AVWVIEW.clim);
        
        handles.sagittal_axes = gca;
        set(gca,'YDir','normal','XLimMode','manual','YLimMode','manual',...
            'ClimMode','manual','YColor',[0, 0, 0],'XColor',[0, 0, 0])
        
        xlabel('Y')
        ylabel('Z')
        title('Sagittal','Color',[0, 0, 0])
        
        % This callback navigates with mouse click
        set(handles.sagittal_image,'ButtonDownFcn','avw_view([],[],''sagittal_image'');');
        
        GUIheight = 0.46;
        
        if AVWVIEW.xdim > 1,
            slider_step(1) = 1/(AVWVIEW.xdim);
            slider_step(2) = 1/(AVWVIEW.xdim);
            handles.sagittal_slider = uicontrol('Parent',GUI,'Style','slider',...
                'Units','Normalized', Font, ...
                'Position',[.55+0.05 GUIheight .113 0.0313], 'HorizontalAlignment', 'center',...
                'BusyAction','cancel',...
                'TooltipString','Sagittal slice navigation',...
                'Min',1,'Max',AVWVIEW.xdim,'SliderStep',slider_step,'Value',AVWVIEW.slices.sag,...
                'Callback','avw_view([],[],''sagittal_slider'');');
        end
        handles.sagittal_sliderN = uicontrol('Parent',GUI,'Style','text',...
            'Units','Normalized', Font, ...
            'Position',[.67+0.05 GUIheight .045 0.0313], 'HorizontalAlignment', 'center',...
            'BusyAction','cancel',...
            'TooltipString','Sagittal slice number',...
            'String',num2str(AVWVIEW.slices.sag),'Value',AVWVIEW.slices.sag);
        handles.sagittal_sliderT = uicontrol('Parent',GUI,'Style','text',...
            'Units','Normalized', Font, ...
            'Position',[.72+0.05 GUIheight .08 0.0313], 'HorizontalAlignment', 'left',...
            'BusyAction','cancel',...
            'TooltipString','Sagittal slice navigation',...
            'String','Sagittal');
        
        handles.sagittal_xlim = get(handles.sagittal_axes,'Xlim');
        handles.sagittal_ylim = get(handles.sagittal_axes,'Ylim');
        handles.sagittal_xline = line('Xdata',[AVWVIEW.slices.cor AVWVIEW.slices.cor],'Ydata',handles.sagittal_ylim);
        handles.sagittal_yline = line('Ydata',[AVWVIEW.slices.axi AVWVIEW.slices.axi],'Xdata',handles.sagittal_xlim);
        set(handles.sagittal_xline,'Color','b','EraseMode','xor','Tag','sagittalXline');
        set(handles.sagittal_yline,'Color','b','EraseMode','xor','Tag','sagittalYline');
        
    end
    
    
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Image Contex Menus
    
    menu=uicontextmenu;
    
    % enable right click access to ROI tools
    % roi = uimenu(menu,'Label','ROI');
    % uimenu(roi,'Label','ROI (9x9x9 block)','Callback','avw_view([],[],''roi_9'');');
    % uimenu(roi,'Label','ROI (7x7x7 block)','Callback','avw_view([],[],''roi_7'');');
    % uimenu(roi,'Label','ROI (5x5x5 block)','Callback','avw_view([],[],''roi_5'');');
    % uimenu(roi,'Label','ROI (3x3x3 block)','Callback','avw_view([],[],''roi_3'');');
    
    % zoom image to new figure
    uimenu(menu,'Label','Zoom Image','Callback','avw_view([],[],''zoom'');');
    uimenu(menu,'Label','Magnify On/Off','Callback','avw_view([],[],''magnify'');');
    
    
    if isfield(handles,'axial_image'),
        if isempty(get(handles.axial_image,'uicontextmenu')),
            set(handles.axial_image,'uicontextmenu',menu);
        end
    end
    if isfield(handles,'coronal_image'),
        if isempty(get(handles.coronal_image,'uicontextmenu')),
            set(handles.coronal_image,'uicontextmenu',menu);
        end
    end
    if isfield(handles,'sagittal_image'),
        if isempty(get(handles.sagittal_image,'uicontextmenu')),
            set(handles.sagittal_image,'uicontextmenu',menu);
        end
    end
    
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    % Image Intensity at Mouse Click
    
    GUIheight = 0.34;
    
    handles.Timpos = uicontrol('Parent',GUI,'Style','text','Units','Normalized', Font, ...
        'Position',[.55+0.05 GUIheight .043 0.0313], 'HorizontalAlignment', 'left',...
        'BusyAction','queue',...
        'String','Voxel');
    handles.impos = uicontrol('Parent',GUI,'Style','edit','Units','Normalized', Font, ...
        'Position',[0.65+0.0 GUIheight .33 0.0313], 'HorizontalAlignment', 'left',...
        'TooltipString','Location in image',...
        'BusyAction','queue',...
        'String','[0 0 0]','Callback','avw_view([],[],''Voxel_Coordinates'');');%
    
    GUIheight = GUIheight - 0.04;
    
    handles.Timval = uicontrol('Parent',GUI,'Style','text','Units','Normalized', Font, ...
        'Position',[.55+0.05 GUIheight .043 0.0313], 'HorizontalAlignment', 'left',...
        'BusyAction','queue',...
        'String','Value');% set width
    handles.imval = uicontrol('Parent',GUI,'Style','edit','Units','Normalized', Font, ...
        'Position',[0.65+0.0 GUIheight .15 0.0313], 'HorizontalAlignment', 'right',...
        'BusyAction','queue',...
        'String','x','Value',0);
    handles.imvalscaled = uicontrol('Parent',GUI,'Style','edit','Units','Normalized', Font, ...
        'Position',[0.65+0.16 GUIheight .17 0.0313], 'HorizontalAlignment', 'right',...
        'BusyAction','queue',...
        'String','x','Value',0);
    
    handles.Tbgimgfrac = uicontrol('Parent',GUI,'Style','text','Units','Normalized', Font, ...
        'Position',[.55+0.05 GUIheight-.04 .265 0.0313], 'HorizontalAlignment', 'left',...
        'BusyAction','queue',...
        'String','Background image overlay fraction');
    handles.bgimgfrac = uicontrol('Parent',GUI,'Style','edit','Units','Normalized', Font, ...
        'Position',[0.82+0.05 GUIheight-.04 .11 0.0313], 'HorizontalAlignment', 'right',...
        'BusyAction','queue',...
        'String',num2str(AVWVIEW.bgimg_fraction),'Value',AVWVIEW.bgimg_fraction,'Callback','avw_view([],[],''bgimgfrac'');');
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    GUIheight = GUIheight - 0.04;
    
    % handles.flip = uicontrol('Parent',GUI,'Style','pushbutton','Units','Normalized', Font, ...
    %     'Position',[.55 GUIheight .10 .03], 'HorizontalAlignment', 'left',...
    %     'BusyAction','queue',...
    %     'TooltipString','Flip Left and Right (viewer only, see also avw_flip).',...
    %     'String','Flip L/R',...
    %     'Callback','avw_view([],[],''flip'');');
    % handles.flipStatus = uicontrol('Parent',GUI,'Style','text','Units','Normalized', Font, ...
    %     'Position',[.65 GUIheight .15 .03], 'HorizontalAlignment', 'right',...
    %     'BusyAction','queue',...
    %     'TooltipString','Flipped Status',...
    %     'String','R>>L (radiological)');
    
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    % AC Location
    
    GUIheight = GUIheight - 0.04;
    
    % handles.Tac = uicontrol('Parent',GUI,'Style','pushbutton','Units','Normalized', Font, ...
    %     'Position',[.55 GUIheight .10 .03], 'HorizontalAlignment', 'left',...
    %     'BusyAction','queue',...
    %     'TooltipString','AC point in (voxels) or (meter offset from center of volume)',...
    %     'String','AC Point',...
    %     'Callback','avw_view([],[],''ac'');');
    % handles.ac = uicontrol('Parent',GUI,'Style','text','Units','Normalized', Font, ...
    %     'Position',[.65 GUIheight .15 .03], 'HorizontalAlignment', 'right',...
    %     'BusyAction','queue',...
    %     'TooltipString','These values are offset from volume center.',...
    %     'String','x,y,z');
    
    % Nasion Location
    
    GUIheight = GUIheight - 0.04;
    
    % handles.Tnasion = uicontrol('Parent',GUI,'Style','pushbutton','Units','Normalized', Font, ...
    %     'Position',[.55 GUIheight .10 .03], 'HorizontalAlignment', 'left',...
    %     'BusyAction','queue',...
    %     'TooltipString','Update Nasion - should be toward +Y',...
    %     'String','Fiducial: Nas',...
    %     'Callback','avw_view([],[],''nasion'');');
    % handles.nasion = uicontrol('Parent',GUI,'Style','text','Units','Normalized', Font, ...
    %     'Position',[.65 GUIheight .15 .03], 'HorizontalAlignment', 'right',...
    %     'BusyAction','queue',...
    %     'TooltipString','These values are offset from volume center, should be toward +Y',...
    %     'String','x,y,z');
    
    % Right Preauricular Location
    
    GUIheight = GUIheight - 0.04;
    
    % handles.Trpa = uicontrol('Parent',GUI,'Style','pushbutton','Units','Normalized', Font, ...
    %     'Position',[.55 GUIheight .10 .03], 'HorizontalAlignment', 'left',...
    %     'BusyAction','queue',...
    %     'TooltipString','Update Right Preauricular - should be toward -X',...
    %     'String','Fiducial: RPA',...
    %     'Callback','avw_view([],[],''rpa'');');
    % handles.rpa = uicontrol('Parent',GUI,'Style','text','Units','Normalized', Font, ...
    %     'Position',[.65 GUIheight .15 .03], 'HorizontalAlignment', 'right',...
    %     'BusyAction','queue',...
    %     'TooltipString','These values are offset from volume center, should be toward -X',...
    %     'String','x,y,z');
    
    % Left Preauricular Location
    
    GUIheight = GUIheight - 0.04;
    
    % handles.Tlpa = uicontrol('Parent',GUI,'Style','pushbutton','Units','Normalized', Font, ...
    %     'Position',[.55 GUIheight .10 .03], 'HorizontalAlignment', 'left',...
    %     'BusyAction','queue',...
    %     'TooltipString','Update Left Preauricular - should be toward +X',...
    %     'String','Fiducial: LPA',...
    %     'Callback','avw_view([],[],''lpa'');');
    % handles.lpa = uicontrol('Parent',GUI,'Style','text','Units','Normalized', Font, ...
    %     'Position',[.65 GUIheight .15 .03], 'HorizontalAlignment', 'right',...
    %         'BusyAction','queue',...
    %     'TooltipString','These values are offset from volume center, should be toward +X',...
    %     'String','x,y,z');
    
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    GUIheight = GUIheight - 0.04;
    
    % handles.contrast = uicontrol('Parent',GUI,'Style','pushbutton','Units','Normalized', Font, ...
    %     'Position',[.55 GUIheight .10 .03], 'HorizontalAlignment', 'left',...
    %     'BusyAction','queue',...
    %     'TooltipString','Auto contrast with gray colormap',...
    %     'String','Auto Contrast',...
    %     'Callback','avw_view([],[],''contrast'');');
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    GUIheight = GUIheight - 0.04;
    
    % handles.dimmer = uicontrol('Parent',GUI,'Style','pushbutton','Units','Normalized', Font, ...
    %     'Position',[.55 GUIheight .10 .03], 'HorizontalAlignment', 'left',...
    %      'BusyAction','queue',...
    %     'TooltipString','Dim by 1%',...
    %     'String','Dimmer',...
    %     'Callback','avw_view([],[],''dimmer'');');
    
    % handles.brighter = uicontrol('Parent',GUI,'Style','pushbutton','Units','Normalized', Font, ...
    %     'Position',[.65 GUIheight .10 .03], 'HorizontalAlignment', 'left',...
    %     'BusyAction','queue',...
    %     'TooltipString','Brighten by 1%',...
    %     'String','Brighter',...
    %     'Callback','avw_view([],[],''brighter'');');
    
    % handles.clim = uicontrol('Parent',GUI,'Style','edit','Units','Normalized', Font, ...
    %     'Position',[.75 GUIheight .06 .03], 'HorizontalAlignment', 'right',...
    %     'BusyAction','queue',...
    %     'TooltipString','Image intensity Climit (inverse brightness)',...
    %     'String',num2str(AVWVIEW.clim(2)),...
    %     'Callback','avw_view([],[],''setClimit'');');
    handles.climtxt = uicontrol('Parent',GUI,'Style','text','Units','Normalized', Font, ...
        'Position',[.55+0.05 0.07 .09613 0.0313], 'HorizontalAlignment', 'left',...
        'BusyAction','queue',...
        'String','Map limits');
    handles.clim = uicontrol('Parent',GUI,'Style','edit','Units','Normalized', Font, ...
        'Position',[ .55+0.05+.09613+0.01 0.07 .276 0.0313], 'HorizontalAlignment', 'right',...
        'BusyAction','queue',...
        'TooltipString','Image color limits',...
        'String',[' [' num2str(AVWVIEW.clim(1)),', ',num2str(AVWVIEW.clim(2)) '] '],...
        'Callback','avw_view([],[],''setClimit'');');
    
    handles.cmaptxt = uicontrol('Parent',GUI,'Style','text','Units','Normalized', Font, ...
        'Position',[.55+0.05 0.03 .09613 0.0313], 'HorizontalAlignment', 'left',...
        'BusyAction','queue',...
        'String','Color map ');
    handles.cmap = uicontrol('Parent',GUI,'Style','popup','Units','Normalized', Font, ...
        'Position',[.55+0.05+.09613+0.01 0.03 .09613 0.0313], 'HorizontalAlignment', 'left',...
        'BusyAction','queue',...
        'TooltipString','Color Map',...
        'String',{'jet','hsv','bwr','gray','bone','maxcon','cadcolors2','bwr13','copper','hot','cool','spring','summer','autumn','winter'},...
        'Callback','avw_view([],[],''setCmap'');');
    
    if ~isempty(AVWVIEW.pois)
        pois=AVWVIEW.pois;
        for i=1:length(pois)
            Name_tags{i} =get(AVWVIEW.pois(i),'name_tag');
        end
        handles.listbox_poi = uicontrol(...
            'Parent',GUI,...
            'Units','Normalized',Font, ...
            'Callback','avw_view([],[],''poi'');',...
            'Position',[0.55+0.05 0.25-0.13 0.38 0.1213],...
            'String',Name_tags,...
            'Style','listbox',...
            'Value',1);
    end
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    GUIheight = 0.46;
    
    handles.crosshairs = uicontrol('Parent',GUI,'Style','checkbox','Units','Normalized', Font, ...
        'Position',[.82+0.05 GUIheight .11 0.0313], 'HorizontalAlignment', 'left',...
        'BusyAction','queue',...
        'TooltipString','Toggle Crosshairs on/off',...
        'String','Crosshairs','Value',1,...
        'Callback','avw_view([],[],''crosshairs'');');
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    GUIheight = GUIheight - 0.04;
    
    % handles.histogram = uicontrol('Parent',GUI,'Style','pushbutton','Units','Normalized', Font, ...
    %     'Position',[.85 GUIheight .10 .03], 'HorizontalAlignment', 'left',...
    %     'BusyAction','queue',...
    %     'TooltipString','Histogram of Volume Intensity',...
    %     'String','Histogram',...
    %     'Callback','avw_view([],[],''histogram'');');
    
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    GUIheight = GUIheight - 0.04;
    
    handles.coord = uicontrol('Parent',GUI,'Style','popupmenu','Units','Normalized', Font, ...
        'Position',[.82+0.05 GUIheight .11 0.0313], 'HorizontalAlignment', 'left',...
        'BusyAction','queue',...
        'TooltipString','Voxel or Mensurated Axis Coordinates',...
        'String',{'Voxels','mm','meters'},...
        'Callback','avw_view([],[],''coordinates'');');
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    Font.FontWeight = 'bold';
    
    % View avw.hdr
    handles.Bhdr = uicontrol('Parent',GUI,'Style','pushbutton','Units','Normalized', Font, ...
        'Position',[.84 .01 .07 0.0313],...
        'String','HDR','BusyAction','queue',...
        'TooltipString','View the .hdr parameters.',...
        'HorizontalAlignment', 'center',...
        'Callback',strcat('AVWVIEW = get(gcbf,''Userdata''); ',...
        'avw_view_hdr(AVWVIEW.avw,AVWVIEW.gui);',...
        'clear AVWVIEW;'));
    
    % OK: Return the avw!
    handles.Bquit = uicontrol('Parent',GUI,'Style','pushbutton','Units','Normalized', Font, ...
        'Position',[.92 .01 .07 0.0313],...
        'String','RETURN','BusyAction','queue',...
        'HorizontalAlignment', 'center',...
        'Callback','avw_view([],[],''quit'');');
    
    handles.colorbar_subplot = subplot('position',[0.545  0.03  0.023 0.461]);
    colorData=repmat(linspace(double(AVWVIEW.clim(2)),double(AVWVIEW.clim(1)), 250)',1,5);
    handles.colorbar_image = imagesc(colorData);
    set(gca,'YLimMode','manual','YTick', [1, 250],...
        'XLimMode','manual','XTick', [0],...
        'YColor',[0, 0, 0],'XColor',[0, 0, 0],'Box','on')
    % set(handles.colorbar_subplot,'CData',);
    
    Modification_interface =true;
    if (Modification_interface)
        filemenu = uimenu('Label','Modify data');
        uimenu(filemenu,'Label','Fill','Callback',@FillFcn,'Accel','T');
        %         uimenu(filemenu,'Label','Flood Fill','Callback',@FloodFillFcn,'Accel','F');
        uimenu(filemenu,'Label','Flood Fill','Callback',@FloodFillFcn,'Accel','F');
        uimenu(filemenu,'Label','2D Flood Fill','Callback',@Flood2DFillFcn,'Accel','2');
        uimenu(filemenu,'Label','Set fill value','Callback',@FillValueFcn);
        uimenu(filemenu,'Label','Save','Callback',@SaveFcn,'Accel','S');
    end
    
    
    % Update the gui_struct handles for this gui
    % if exist('parent','var'), %Petr Krysl
    %     AVWVIEW.parent.gui = parent;
    %     ud=set(parent,'Userdata');
    %     ud.current_avw_gui =GUI;
    %     set(parent,'Userdata',ud);
    % end
    AVWVIEW.avw = avw;
    AVWVIEW.handles = handles;
    set(AVWVIEW.gui,'Userdata',AVWVIEW);
    
    AVWVIEW = set_coordinates(AVWVIEW);
    set(AVWVIEW.gui,'HandleVisibility','callback');
    
    draw_colorbar(AVWVIEW)
    
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function slice_img(avw),
    
    figure
    xslice = 128;
    slice = squeeze( avw.img(xslice,:,:) );
    imagesc(slice); axis image; colormap('gray')
    figure
    yslice = 128;
    slice = squeeze( avw.img(:,yslice,:) );
    imagesc(slice); axis image; colormap('gray')
    figure
    zslice = 128;
    slice = squeeze( avw.img(:,:,zslice) );
    imagesc(slice); axis image; colormap('gray')
    
end

function color=map2color_method(colormap, clim, values)
    color=mapvalues2color(colormap, clim, values);
end

function  button_sagittal(src,evnt)
    
    if evnt.Character == 'e'
        %                 print ('-deps',['-f' num2str(src)])
    elseif length(evnt.Modifier) == 1 & strcmp(evnt.Modifier{:},'control') & evnt.Key == 't'
        %                 print ('-dtiff','-r200',['-f' num2str(src)])
    end
end
%     cbh=colorbar('Position',position,'CLimMode','auto','YLim',minmax,'YLimMode','manual');
%     YTick= get(cbh,'YTick')
% %     set(cbh,'Position',position,'YLim',minmax);
%     YTick= get(cbh,'YTick');
% %     YTick=(YTick-minmax(1))./(minmax(2)-minmax(1));
%     YTickLabel= get(cbh,'YTickLabel');
%     m=size(YTickLabel,1);
%     yl(1:m)=deal({' '});
%     for i=1:m
%         yl(i)={YTickLabel(i,:)};
%     end
%     YTickLabel=yl;
%     mtd=(minmax(2)-minmax(1))/5/length(YTick);
%     if abs(YTick(1)-minmax(1)) >mtd
%         YTick=[minmax(1),YTick];
%         YTickLabel=cat(2,{num2str(minmax(1))},YTickLabel);
%     end
%     if abs(YTick(end)-minmax(2)) >mtd
%         YTick=[YTick,minmax(2)];
%         YTickLabel= cat(2,YTickLabel,{num2str(minmax(2))});
%     end
%     set(cbh,'YTick', YTick,'YTickLabel',YTickLabel);
%  set(cbh,'YLimMode','auto')
%     set(get(cbh,'XLabel'),'String',label);
%  colormap(cmap);


function draw_colorbar(AVWVIEW)
    colorData=repmat(linspace(double(AVWVIEW.clim(2)),double(AVWVIEW.clim(1)), 250)',1,5);
    color=map2color_method(AVWVIEW.cmap, AVWVIEW.clim,colorData);
    set(AVWVIEW.handles.colorbar_image,'CData',reshape(color,[size(colorData),3]));
    set(AVWVIEW.handles.colorbar_subplot,'YLimMode','manual','YTick', [1, 250],...
        'YTickLabel',cat(2,{num2str(AVWVIEW.clim(2))},{num2str(AVWVIEW.clim(1))}));
    return
end

function SaveFcn(hFig,event_data)
    if gcbf,
        AVWVIEW = get(gcbf,'Userdata');
    end
    avw_write(AVWVIEW.avw,[AVWVIEW.avw.fileprefix '-backup']);
    set(AVWVIEW.gui,'Userdata',AVWVIEW);
end

function FillValueFcn(hFig,event_data)
    if gcbf,
        AVWVIEW = get(gcbf,'Userdata');
    end
    answer = inputdlg(' Input fill value');
    try
        AVWVIEW.fill_value = eval(answer{1});
    catch
        AVWVIEW.fill_value = 0;
    end
    answer = inputdlg(' Input flag value');
    try
        AVWVIEW.flag_value = eval(answer{1});
    catch
        AVWVIEW.flag_value = intmin(class(AVWVIEW.avw.img));
    end
    set(AVWVIEW.gui,'Userdata',AVWVIEW);
end

function FillFcn(hFig,event_data)
    if gcbf,
        AVWVIEW = get(gcbf,'Userdata');
    end
    try
        [i,j,k] =adeal(get(AVWVIEW.handles.impos,'Value'));
        AVWVIEW.avw.img(i,j,k)=AVWVIEW.fill_value;
    catch,end
    set(AVWVIEW.gui,'Userdata',AVWVIEW);
    AVWVIEW.view = 'axi';
    axial_update(AVWVIEW);
    AVWVIEW.view = 'cor';
    coronal_update(AVWVIEW);
    AVWVIEW.view = 'sag';
    sagittal_update(AVWVIEW);
    AVWVIEW = set_coordinates(AVWVIEW);
end


function FloodFillFcn(hFig,event_data)
    if gcbf,
        AVWVIEW = get(gcbf,'Userdata');
    end
    try
        [i,j,k] =adeal(get(AVWVIEW.handles.impos,'Value'));
        if (AVWVIEW.avw.img(i,j,k)~=AVWVIEW.fill_value)
            Original_img =AVWVIEW.avw.img;
            AVWVIEW.avw.img =flood_fill(AVWVIEW.avw.img,i,j,k,AVWVIEW.flag_value,AVWVIEW.avw.img(i,j,k));
            set(AVWVIEW.gui,'Userdata',AVWVIEW);
            AVWVIEW.view = 'axi';
            axial_update(AVWVIEW);
            AVWVIEW.view = 'cor';
            coronal_update(AVWVIEW);
            AVWVIEW.view = 'sag';
            sagittal_update(AVWVIEW);
            AVWVIEW = set_coordinates(AVWVIEW);
            drawnow;
            button = questdlg(' Accept fill?','Accept fill?','Okay','Cancel','Cancel');
            if (strcmp( button,'Okay'))
                AVWVIEW.avw.img =Original_img;
                AVWVIEW.avw.img =flood_fill(AVWVIEW.avw.img,i,j,k,AVWVIEW.fill_value,AVWVIEW.avw.img(i,j,k));
            else
                AVWVIEW.avw.img =Original_img;
            end
            set(AVWVIEW.gui,'Userdata',AVWVIEW);
            AVWVIEW.view = 'axi';
            axial_update(AVWVIEW);
            AVWVIEW.view = 'cor';
            coronal_update(AVWVIEW);
            AVWVIEW.view = 'sag';
            sagittal_update(AVWVIEW);
            AVWVIEW = set_coordinates(AVWVIEW);
        end
    catch,end
    
end

function Flood2DFillFcn(hFig,event_data)
    if gcbf,
        AVWVIEW = get(gcbf,'Userdata');
    end
    try
        [i,j,k] =adeal(get(AVWVIEW.handles.impos,'Value'));
        if (AVWVIEW.avw.img(i,j,k)~=AVWVIEW.fill_value)
            Original_img =AVWVIEW.avw.img;
            switch AVWVIEW.view
                case 'axi'
                    AVWVIEW.avw.img(1:end,1:end,k) =...
                        flood_fill_no_restart(AVWVIEW.avw.img(1:end,1:end,k),i,j,1,AVWVIEW.flag_value,AVWVIEW.avw.img(i,j,k));
                case 'cor'
                    AVWVIEW.avw.img(1:end,j,1:end) =...
                        flood_fill_no_restart(AVWVIEW.avw.img(1:end,j,1:end),i,1,k,AVWVIEW.flag_value,AVWVIEW.avw.img(i,j,k));
                case 'sag'
                    AVWVIEW.avw.img(i,1:end,1:end) =...
                        flood_fill_no_restart(AVWVIEW.avw.img(i,1:end,1:end),1,j,k,AVWVIEW.flag_value,AVWVIEW.avw.img(i,j,k));
            end
            set(AVWVIEW.gui,'Userdata',AVWVIEW);
            AVWVIEW.view = 'axi';
            axial_update(AVWVIEW);
            AVWVIEW.view = 'cor';
            coronal_update(AVWVIEW);
            AVWVIEW.view = 'sag';
            sagittal_update(AVWVIEW);
            AVWVIEW = set_coordinates(AVWVIEW);
            drawnow;
            button = questdlg(' Accept fill?','Accept fill?','Okay','Cancel','Cancel');
            if (strcmp( button,'Okay'))
                %                 AVWVIEW.avw.img =Original_img;
                AVWVIEW.avw.img(find(AVWVIEW.avw.img==AVWVIEW.flag_value)) =AVWVIEW.fill_value;
            else
                AVWVIEW.avw.img =Original_img;
            end
            set(AVWVIEW.gui,'Userdata',AVWVIEW);
            AVWVIEW.view = 'axi';
            axial_update(AVWVIEW);
            AVWVIEW.view = 'cor';
            coronal_update(AVWVIEW);
            AVWVIEW.view = 'sag';
            sagittal_update(AVWVIEW);
            AVWVIEW = set_coordinates(AVWVIEW);
        end
    catch,end
    
end

function i_M_g =flood_fill_no_restart(i_M_g,xc,yc,zc,new,old)
    [width, height, depth] = size(i_M_g);
    
    call=Inf;
    maxcall=480;
    have_next=~true;
    
    function do_fill(x,y,z)
        call = call + 1;
        if (x<2 || x>=width)
            x = xc;
        end
        if (y<2 || y>=height)
            y = yc;
        end
        if (z<2 || z>=depth)
            z = zc;
        end
        
        if call>maxcall
            call =Inf;
            if (i_M_g(x,y,z) == old) && (~have_next)
                xc=x; yc=y; zc=z;
                have_next=true;
            end
            return;
        else
            if (i_M_g(x,y,z) == old)
                i_M_g(x,y,z)= new;
                do_fill(x+1,y,z);
                do_fill(x,y,z+1);
                do_fill(x,y+1,z);
                do_fill(x,y-1,z);
                do_fill(x,y,z-1);
                do_fill(x-1,y,z);
            end
        end
    end
    
    while true
        tic;
        call=1;
        have_next=~true;
        do_fill(xc,yc,zc);
        if ~have_next
            break;
        end
        toc
    end
    
end

function color=mapvalues2color(colormap, clim, values)
    ncolors=size(colormap,1);
    clim =double([min(clim),max(clim)]);
    if ((clim(2)-clim(1))==0)
        k = fix(((values-clim(1))*0)*(ncolors-1))+1;
    else
        k = fix(((values-clim(1))/(clim(2)-clim(1)))*(ncolors-1))+1;
    end
    kl= find(k < 1);
    if ~isempty(kl)
        k(kl)=1;
    end
    kl= find(k > ncolors);
    if ~isempty(kl)
        k(kl)=ncolors;
    end
    color=colormap(k,:);
    return
end